/* Name: hidtool.c
 * Project: Basic USB thermometer based on hid-data example from avrusb project.
 * Author: Russel McAuliffe
 * Creation Date: 2008-06-11
 * Copyright: (c) 2008 by sentientresearch ltd
 * License: GNU GPL v2 (see License.txt) or proprietary (CommercialLicense.txt)
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <windows.h>
#include <time.h>

#include "hiddata.h"
#include "../firmware/usbconfig.h"  /* for device VID, PID, vendor name and product name */

typedef struct 	
{
  int bucket[1024];
  int bestscore;
  int bestbucket;
  int total;
} measurement;

measurement measurements[4];

#define NumChecks 5  // number of readings to take for average and mode measurements

int bucket[1024]; 


/* ------------------------------------------------------------------------- */

// ***************************************
static char *usbErrorMessage(int errCode)
// ***************************************
{
static char buffer[80];

    switch(errCode){
        case USBOPEN_ERR_ACCESS:      return "Access to device denied";
        case USBOPEN_ERR_NOTFOUND:    return "The specified device was not found";
        case USBOPEN_ERR_IO:          return "Communication error with device";
        default:
            sprintf(buffer, "Unknown USB error %d", errCode);
            return buffer;
    }
    return NULL;    /* not reached */
}

// ***************************************
static usbDevice_t  *openDevice(void)
// ***************************************
{
usbDevice_t     *dev = NULL;
unsigned char   rawVid[2] = {USB_CFG_VENDOR_ID}, rawPid[2] = {USB_CFG_DEVICE_ID};
char            vendorName[] = {USB_CFG_VENDOR_NAME, 0}, productName[] = {USB_CFG_DEVICE_NAME, 0};
int             vid = rawVid[0] + 256 * rawVid[1];
int             pid = rawPid[0] + 256 * rawPid[1];
int             err;

    if((err = usbhidOpenDevice(&dev, vid, vendorName, pid, productName, 0)) != 0){
        fprintf(stderr, "error finding %s: %s\n", productName, usbErrorMessage(err));
        return NULL;
    }
    return dev;
}



// To do the mode, each number read is thrown into it's bucket then at the end the buckets are counted up

// ***************************************
void EmptyBuckets(void) 
// ***************************************
{
  int x,y;
  for(y=0;y<4;y++) {

    measurements[y].bestscore=0;
    measurements[y].bestbucket=-2;
    measurements[y].total=0;
    
    for(x=0;x<1024;x++) {
      measurements[y].bucket[x]=0;           
    }
  }
} 

// ***************************************
static void TallyBuckets(char *buffer, int len) // throw last measurement for each input into its respective bucket
// ***************************************

{
  int i;
  int b;

  for(i=0;i<8;i+=2) {
  
    b=((buffer[i] & 0xff) << 8)  + (buffer[i+1] & 0xff);  // join the two bytes (right justified) into a 10 bit number
    
    measurements[i/2].bucket[b]++;  // increment the bucket - bucket with most in at the end is the mode    
    measurements[i/2].total+=b;
  }  
}


// ***************************************
void GetBestBuckets(void)
// ***************************************
{

  int x,y;
  int highestCount;
  int bestBucket;



  for(y=0;y<4;y++) {  // four thermistor inputs
    measurements[y].bestscore=0;
    measurements[y].bestbucket=-1;
    for(x=0;x<1024;x++) {  // go through all the buckets to see which was best. not very efficient :)
      if(measurements[y].bucket[x]>measurements[y].bestscore) {
        measurements[y].bestbucket=x;  // best bucket for this input
        measurements[y].bestscore=measurements[y].bucket[x]; // the number of items in the bucket
      }
    }
  }
  return;
}

// ***************************************
float RawToDegrees (int raw) 
// ***************************************
{
  float degrees;
  float fint;
  fint=raw;
//  degrees = (0.2693 * fint) - 191.81;

//  degrees = (0.3229 * fint) - 156.54; // 2.5k resistor in divider  
  degrees = (0.3053 * fint) - 164.6; // 2.2k resistor in divider  


  return(degrees);
	  
}

// ***************************************
void bin_prnt_byte(int x)  // just for debugging
// ***************************************
{
   int n;
   for(n=0; n<8; n++)
   {
      if((x & 0x80) !=0)
      {
         printf("1");
      }
      else
      {
         printf("0");
      }
      if (n==3)
      {
         printf(" "); /* insert a space between nybbles */
      }
      x = x<<1;
   }
}

// ***************************************
void bin_prnt_int(int x) // just for debugging
// ***************************************
{
   int hi, lo;
   hi=(x>>8) & 0xff;
   lo=x&0xff;
   bin_prnt_byte(hi);
   printf(" ");
   bin_prnt_byte(lo);
}




// ***************************************
static int  hexread(char *buffer, char *string, int buflen)
// ***************************************
{
char    *s;
int     pos = 0;

    while((s = strtok(string, ", ")) != NULL && pos < buflen){
        string = NULL;
        buffer[pos++] = (char)strtol(s, NULL, 0);
    }
    return pos;
}

/* ------------------------------------------------------------------------- */

// ***************************************
static void usage(char *myName)
// ***************************************
{
    fprintf(stderr, "usage:\n");
    fprintf(stderr, "  %s read\n", myName);
    fprintf(stderr, "  %s write <listofbytes>\n", myName);
}

// ***************************************
int main(int argc, char **argv)
// ***************************************
{
  usbDevice_t *dev;
  char        buffer[129];    /* room for dummy report ID */
  int         err;
  int x;

  if((dev = openDevice()) == NULL) exit(1);

  EmptyBuckets();
  for (x=0;x<NumChecks;x++) {          
    int len = sizeof(buffer);
    if((err = usbhidGetReport(dev, 0, buffer, &len)) != 0){
        fprintf(stderr, "error reading data: %s\n", usbErrorMessage(err));
        exit(0);
    }else{
       TallyBuckets(buffer + 1, sizeof(buffer) - 1); // sort the values into their buckets
    }
    Sleep(10);        
  }             
  GetBestBuckets();  // get most commonly returned reading for each input


  // ** Report results using most frequently read value for each input
  for(x=0;x<4;x++) {
//    printf("%.1f (%d) ",RawToDegrees(measurements[x].bestbucket),measurements[x].bestbucket);
    printf("%.1f ",RawToDegrees(measurements[x].bestbucket));
  }	
  printf("\n");

  

  // ** Report results showing average score for each input  
  /*
  for(x=0;x<4;x++) {
    printf("%.1f ",RawToDegrees(measurements[x].total/NumChecks));
  }	
  printf("\n");  */
  
    
  usbhidCloseDevice(dev);
  return 0;
}

/* ------------------------------------------------------------------------- */

